/*
 * @(#)AList 31 October 2006
 *
 * @author Viera K. Proulx
 */
/**
 * A class that implements several traversal-based algorithms, such as 
 * filter and contains.
 */
public class Algorithms {
  
  /**
   * Cons onto the given list of all elements generated 
   * by the given traversal that satisfy the given predicate.
   * 
   * @param tr the given traversal
   * @param alist the given list (accumulator)
   * @param choice the given predicate
   */
  public static <T extends ISame<T>> AList<T> filter(Traversal<T> tr, 
           AList<T> alist, 
           ISelect<T> choice){
    return alist; // a place holder for the correct method
    // return 'null' when the exception is caught.
  }
  
  /**
   * Determine if the collection generated by the given traversal 
   * contains an element that satisfies the given predicate.
   * 
   * @param tr the given traversal
   * @param choice the given predicate
   */
  public static <T extends ISame> boolean hasSuch(Traversal<T> tr, ISelect<T> choice){
    try{
      if (tr.isEmpty())
        return false;
      else    
        if (choice.select(tr.getFirst()))
          return true;
        else
          return hasSuch(tr.getRest(), choice); 
    }
    catch(IllegalUseOfTraversalException e){
      System.out.println("Illegal traversal: " + e.getMessage());
      return false;
    }
  }
}