/*
 * @(#)Balloon 31 October 2006
 *
 * @author Viera K. Proulx
 */
import java.awt.*;
import java.util.*;

/**
 * A sample class used to illustrate state change.
 */ 
public class Balloon implements ISame<Balloon>, IChange{
  /**
   * the location of the center of this balloon.
   */
  public CartPt loc;
  /**
   * the radius of this balloon.
   */
  public int radius;
  /**
   * the color of this balloon.
   */
  public Color col;
  
  /**
   * The full constructor.
   * @param loc the location of the center of this balloon.
   * @param radius the radius of this balloon.
   * @param col the color of this balloon.
   */
  public Balloon(CartPt loc, int radius, Color col){
    this.loc = loc;
    this.radius = radius;
    this.col = col;
  }
  
  /**
   * Change the radius of this balloon by the given amount.
   * 
   * @param dr the given difference in radius.
   */ 
  public void inflateBalloon(int dr){
    this.radius = this.radius + dr;  
  }
  
  /**
   * Change the radius of this balloon by the given amount.
   * 
   * @param dr the given difference in radius.
   */ 
  public void change(int dr){
    this.radius = this.radius + dr;  
  }
  
  /**
   * Change the location of this balloon by moving it up the given amount.
   * 
   * @param dy the distance balloon should rise.
   */ 
  public void floatBalloon(int dy){
    this.loc.y = this.loc.y - dy;
  }
  
  /**
   * Is this balloon the same as the given object?
   * 
   * @param obj the object to compare with.
   */ 
  public boolean same(Balloon obj){
    return this.loc.same(obj.loc) &&
    this.radius == obj.radius &&
    this.col.getRed() == obj.col.getRed() &&
    this.col.getGreen() == obj.col.getGreen() &&
    this.col.getBlue() == obj.col.getBlue();
  } 
  
  /**
   * Represent the data in this instance as a <CODE>String</CODE>.
   */ 
  public String toString(){
    return "new Balloon(loc.x = " + this.loc.x + ", loc.y = " + this.loc.y +
    ", radius = " + this.radius + ");";
  }
}

/**
 * Class to represent a location using integer cartesian coordinates.
 */
class CartPt implements ISame{
  /**
   * the horizontal coordinate.
   */
  public int x;
  /**
   * the vertical coordinate.
   */
  public int y;
  
  /**
   * The full constructor.
   * @param x the horizontal coordinate.
   * @param y the vertical coordinate.
   */
  public CartPt(int x, int y){
    this.x = x;
    this.y = y;
  }
  
  /**
   * Is this CartPt the same as the given object?
   * 
   * @param obj the object to compare with.
   */ 
  public boolean same(Object obj){
    if (obj instanceof CartPt)
      return this.x == ((CartPt)obj).x &&
      this.y == ((CartPt)obj).y;
    else 
      return false;
  }
}

/**
 * <CODE>{@link Comparator Comparator}</CODE> 
 * for the <CODE>{@link Balloon Balloon}</CODE> class: 
 * by the radius of the balloon.
 */ 
class BalloonByRadius implements Comparator<Balloon>{
  
  /**
   * Compare two balloons by their radius.
   * 
   * @param b1 the first balloon.
   * @param b2 the second balloon.
   */
  public int compare(Balloon b1, Balloon b2){
    return b1.radius - b2.radius;
  }
}

/**
 * <CODE>{@link Comparator Comparator}</CODE> 
 * for the <CODE>{@link Balloon Balloon}</CODE> class: 
 * by the height of the balloon.
 */ 
class BalloonByHeight implements Comparator<Balloon>{
  
  /**
   * Compare two balloons by their height.
   * 
   * @param b1 the first balloon.
   * @param b2 the second balloon.
   */
  public int compare(Balloon b1, Balloon b2){
    return b2.loc.y - b1.loc.y;
  }
}