/*
 * @(#)ImageFile 31 October 2006
 *
 * @author Viera K. Proulx
 */
import java.util.*;

/**
 * A sample class used to illustrate state change.
 */ 
public class ImageFile implements ISame<ImageFile>, IChange{
  /**
   * the name of this file.
   */
  public String name;
  /**
   * the width of this image.
   */
  public int width;
  /**
   * the height of this image.
   */
  public int height;
  
  /**
   * The full constructor.
   * @param name the name of the file.
   * @param width the width of the image.
   * @param height the height of the image.
   */
  public ImageFile(String name, int width, int height){
    this.name = name;
    this.width = width;
    this.height = height;
  }
  
  /**
   * Crop this image in both directions by the given amount.
   * There is no checking for possible negative size.
   * 
   * @param diff the amount by which both the width 
   * and the height are decreased.
   */
  public void crop(int diff){
    this.width = this.width - diff;  
    this.height = this.height - diff;  
  }
  
  /**
   * Crop this image in both directions by the given amount.
   * 
   * @param diff the amount by which both the width 
   * and the height are decreased.
   */ 
  public void change(int diff){
    this.width = this.width - diff;  
    this.height = this.height - diff;    
  }
  
  /**
   * Is this image file the same as the given object? 
   * 
   * @param obj the object to compare with.
   */ 
  public boolean same(ImageFile obj){
    return this.name.equals(obj.name) &&
    this.width == obj.width &&
    this.height == obj.height;
  } 
  
  /**
   * Represent the data in this instance as a <CODE>String</CODE>.
   */ 
  public String toString(){
    return "new ImageFile(name = " + this.name + ", width = " + this.width +
    ", height = " + this.height + ");";
  }
}

/**
 * <CODE>{@link Comparator Comparator}</CODE> 
 * for the <CODE>{@link ImageFile ImageFile}</CODE> class: 
 * by the name of the image file.
 */ 
class ImageFileByName implements Comparator<ImageFile>{
  
  /**
   * Compare two image files by their name.
   * 
   * @param img1 the first image file.
   * @param img2 the second image file.
   */
  public int compare(ImageFile img1, ImageFile img2){
    return img1.name.compareTo(img2.name);
  }
}

/**
 * <CODE>{@link Comparator Comparator}</CODE> 
 * for the <CODE>{@link ImageFile ImageFile}</CODE> class: 
 * by the size of the image file.
 */ 
class ImageFileBySize implements Comparator<ImageFile>{
  
  /**
   * Compare two image files by their size.
   * 
   * @param img1 the first image file.
   * @param img2 the second image file.
   */
  public int compare(ImageFile img1, ImageFile img2){
    return img1.width * img1.height -
           img2.width * img2.height;
  }
}