/*
 * @(#)MTList 31 October 2006
 *
 * @author Viera K. Proulx
 */
/**
 * A class to represent an empty list of elements
 * of the type <CODE>T</CODE> that also implements the 
 * <CODE>{@link ISame ISame}</CODE> interface.
 * It extends the <CODE>{@link Traversal Traversal}</CODE> interface.
 */
public class MTList<T extends ISame<T>> implements AList<T>{
  
  /**
   * Does this list contain the given object?
   * 
   * @param obj the given object.
   */ 
  public boolean contains(T obj){
    return false; 
  }
  
  /**
   * Count the items in this list.
   */ 
  public int count(){ 
    return 0; 
  }
  
  /** 
   * Throw an exception: The first element cannot be removed.
   * 
   * @return new traversal for a collection without the first element.
   * 
   */
  public Traversal<T> getRest() throws IllegalUseOfTraversalException{
    throw new IllegalUseOfTraversalException("No 'next' in an empty list");
  }
  
  /** 
   * Throw an exception: There is no first element.
   * 
   * @return the first element in the collection.
   */
  public T getFirst() throws IllegalUseOfTraversalException{
    throw new IllegalUseOfTraversalException("No 'first' in an empty list");
  } 
  
  /** 
   * Test whether there are any items in the collection.
   * 
   * @return true if the collection has no elements.
   */
  public boolean isEmpty(){
    return true;
  }
  
  /**
   * Represent this instance as a <CODE>String</CODE>.
   */ 
  public String toString(){
    return "new MTList()";
  }
}
