/*
 * @(#)TopThree 31 October 2006
 *
 * @author Viera K. Proulx
 */
import java.util.*;

/**
 * A class designed to illustrate the use of type parameters
 * and the mutation of the <CODE>{@link ArrayList ArrayList}</CODE>.
 * Data consists of three elements ordered by the given 
 * <CODE>{@link Comparator Comparator}</CODE>.
 */
public class TopThree <T extends IChange & ISame> implements ISame{
 
  /**
   * An <CODE>{@link ArrayList ArrayList}</CODE> to hold the three
   * data elements.
   */
  public ArrayList<T> all;
  /**
   * The comparator that defines the ordering of the elements.
   */
  public Comparator<T> comp;
  
  /**
   * The full constructor.
   */ 
  public TopThree(T one, T two, T three, Comparator<T> comp){
    this.all = new ArrayList<T>(3);
    this.all.add(0, one);
    this.all.add(1, two);
    this.all.add(2, three);
    this.comp = comp;
    this.reorder();
  }
  
  /**
   * Change each element by the given amount and re-order the data.
   */ 
  public void change(int d1, int d2, int d3){
    this.all.get(0).change(d1);
    this.all.get(1).change(d2);
    this.all.get(2).change(d3);
    this.reorder();
  }
  
  /**
   * Reorder the data using the 'resident' <CODE>Comparator</CODE>.
   */
  public void reorder(){
    T temp = this.all.get(0);
    if (this.comp.compare(this.all.get(0), this.all.get(1)) < 0){
      // ...
    } 
    // ...
  }
   
   
  /**
   * Is this TopThree same as the given Object?
   */ 
  public boolean same(Object obj){
    if (obj instanceof TopThree)
      return this.all.get(0).same(((TopThree)obj).all.get(0)) &&
             this.all.get(1).same(((TopThree)obj).all.get(1)) &&
             this.all.get(2).same(((TopThree)obj).all.get(2)) &&
             this.comp.equals(((TopThree)obj).comp);
    else
      throw new ClassCastException(
          "TopThree cannot be compared with " + obj.toString());
  }
  
  /**
   * Represent the data in this instance as a <CODE>String</CODE>.
   */ 
  public String toString(){
    return 
    "new TopThree(\n" +
    "this.numberone = " + this.all.get(0).toString() +",\n" +
    "this.numbertwo = " + this.all.get(1).toString() +",\n" +
    "this.numberthree = " + this.all.get(2).toString() +")\n";
  }
}