/*
 * @(#)Traversal 31 October 2006
 *
 * @author Viera K. Proulx
 */

/**
 * An interface that provides a functional iterator over a collection
 * of data if the type <CODE>T</CODE>.
 * 
 * Type <CODE>T</CODE> must implement the 
 * <CODE>{@link ISame ISame}</CODE> interface
 */
public interface Traversal<T extends ISame>{
  
  /** 
   * Produce a traversal for the collection with the first element removed.
   * Throw an <CODE>{@link IllegalUseOfTraversalException 
   * IllegalUseOfTraversalException}</CODE>  
   * if the first element cannot be removed.
   * 
   * @return new traversal for a collection without the first element.
   * @throws IllegalUseOfTraversalException
   * if the first element cannot be removed.
   */
  public Traversal<T> getRest() throws IllegalUseOfTraversalException; 
  
  /** 
   * Provide the first element of this collection. 
   * Throw an <CODE>{@link IllegalUseOfTraversalException 
   * IllegalUseOfTraversalException}</CODE> if there is no first element. 
   * 
   *  
   * @return the first element in the collection.
   * @throws IllegalUseOfTraversalException
   * if there is no first element.
   */
  public T getFirst() throws IllegalUseOfTraversalException; 
  
  /** 
   * Test whether there are any items in the collection.
   * 
   * @return true if the collection has no elements.
   */
  public boolean isEmpty(); 
}

